/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

AmetysContainer = function (allowDeletion, allowToggle)
{
	this.allowDeletion_ = allowDeletion;
	this.allowToggle_ = allowToggle;
}

AmetysContainer.prototype.allowDeletion_;
AmetysContainer.prototype.allowDeletion = function (allow)
{
	this.allowDeletion_ = allow;
}

AmetysContainer.prototype.allowToggle_;
AmetysContainer.prototype.allowToggle = function (allow)
{
	AmetysContainer.prototype.allowToggle_ = allow;
}

AmetysContainer.prototype.setSettingsButton = function (title, iconPath, useIcon)
{
	this.settingsButtonTitle_ = title;
	this.settingsButtonIconPath_ = iconPath;
	this.settingsButtonUseIcon_ = useIcon;
}

AmetysContainer.prototype.setToggleButton = function (title, iconPath, useIcon)
{
	this.toggleButtonTitle_ = title;
	this.toggleButtonIconPath_ = iconPath;
	this.toggleButtonUseIcon_ = useIcon;
}

AmetysContainer.prototype.setDeleteButton = function (title, iconPath, useIcon)
{
	this.deleteButtonTitle_ = title;
	this.deleteButtonIconPath_ = iconPath;
	this.deleteButtonUseIcon_ = useIcon;
}

AmetysContainer.prototype.userPrefsContext_ = '/sites/' + context.parameters.siteName;
AmetysContainer.prototype.setUserPrefsContext = function (context)
{
	this.userPrefsContext_ = context;
}
AmetysContainer.prototype.getUserPrefsContext = function ()
{
	return this.userPrefsContext_;
}

AmetysContainer.prototype.gadgets_ = {};
AmetysContainer.prototype.count_ = 0;
AmetysContainer.prototype.addGadget = function (gadget)
{
	this.gadgets_[gadget._ametysID] = gadget;
	this.count_++;
}
AmetysContainer.prototype.removeGadget = function (gadgetID)
{
	this.count_--;
	delete this.gadgets_[gadgetID];
}
AmetysContainer.prototype.getCount = function ()
{
	return this.count_;
}
/**
 * Render a gadget into a div
 * @param gadgetUrl the gadget url
 */
AmetysContainer.prototype.addGadgetToContainer = function (gadgetUrl, layout, callback) 
{
	var me = this;
	this.requestGadgetMetaData (gadgetUrl, function (metadata) { me.generateGadgets (metadata, layout, me.allowDeletion_, me.allowToggle_, callback)});
};

AmetysContainer.prototype.registerGadget = function (gadget, rawUrl, netvibesSpecUrl) 
{
	// create unique id for gadget
	gadget._ametysID = this.uniqueID ();
	gadget._gadgetUrl = rawUrl;
	gadget._netvibesSpecUrl = netvibesSpecUrl !=  '' ? netvibesSpecUrl : null;
	
	this.addGadget(gadget);
	
	var params = {};
	params.gadgetId = gadget._ametysID;
	params.userPrefsContext = this.getUserPrefsContext();
	
	var data = {};
	// gadget url
	data.specUrl = gadget.specUrl;
	// gadget default user prefs
	for (var i in gadget.getUserPrefs())
	{
		data[i] = gadget.getUserPrefs()[i].value;
	}
	// gadget position
	data['position_X'] = this.getPositionX (gadget);
	data['position_Y'] = this.getPositionY (gadget);
	data['gadgetUrl'] = gadget.getGadgetUrl();
	if (gadget.getNetvibesSpecUrl() != null)
	{
		data['netvibesSpecUrl'] = gadget.getNetvibesSpecUrl();
	}
	
	params.data = encodeURIComponent(gadgets.json.stringify(data));
	
	$j.ajax({
		  url: getPluginDirectUrl('gadgets') + '/userprefs/save',
		  data: params,
		  success: function(data) {
		  }
	});
};

AmetysContainer.prototype.uniqueID = function () 
{ 
	var uniqueID = new Date();
	return 'gadget_' + uniqueID.getTime() + '_' + String(Math.random()).substring(2);
}

AmetysContainer.prototype.getGadgetFromChrome = function (chromeId)
{
	for (var key in this.gadgets_)
	{
		var gadget = this.gadgets_[key];
		var layout = $j('#' + gadget.getIframeId()).parentsUntil('.gadgets-layout-gadget').parent('.gadgets-layout-gadget')[0];
		
		if (layout.id == chromeId)
		{
			return gadget;
		}
	}
	return null;
}

AmetysContainer.prototype.updatePositions = function (event, ui)
{
	var params = {};
	for (var key in this.gadgets_)
	{
		var gadget = this.gadgets_[key];
		
		var x = this.getPositionX (gadget);
		var y = this.getPositionY (gadget);
		
		params.userPrefsContext = this.getUserPrefsContext();
		var data = {};
		data.specUrl = gadget.specUrl;
		for (var i in gadget.getUserPrefs())
		{
			data[i] = gadget.getUserPrefs()[i].value;
		}
		// gadget position
		data['position_X'] = this.getPositionX (gadget);
		data['position_Y'] = this.getPositionY (gadget);
		data['gadgetUrl'] = gadget.getGadgetUrl();
		if (gadget.getNetvibesSpecUrl() != null)
		{
			data['netvibesSpecUrl'] = gadget.getNetvibesSpecUrl();
		}
		
		params['data_' + gadget._ametysID] = encodeURIComponent(gadgets.json.stringify(data));
	}
	
	$j.ajax({
		  url: getPluginDirectUrl('gadgets') + '/update-position',
		  data: params,
		  success: function(data) {
		  }
	});
}

AmetysContainer.prototype.getPositionX = function (gadget)
{
	var column = $j('#' + gadget.getIframeId()).parentsUntil('.gadgets-layout-column').parent('.gadgets-layout-column')[0];
	if (column != null)
	{
		var index = column.id.lastIndexOf('-');
		return parseInt(column.id.substring(index + 1, column.id.length))
	}
	return 0; 
}

AmetysContainer.prototype.getPositionY = function (gadget)
{
	var gadgetLayout = $j('#' + gadget.getIframeId()).parentsUntil('.gadgets-layout-gadget').parent('.gadgets-layout-gadget')[0];
	var column = $j('#' + gadget.getIframeId()).parentsUntil('.gadgets-layout-column').parent('.gadgets-layout-column');
	var children = column.children('.gadgets-layout-gadget')
	for (var i=0; i < children.length; i++)
	{
		if (children[i].id == gadgetLayout.id)
		{
			return i;
		}
	}
	return 0;
}

AmetysContainer.prototype.initGadget = function (gadget, id, userPrefs, rawUrl, netvibesSpecUrl) 
{
	gadget._ametysID = id;
	gadget._gadgetUrl = rawUrl;
	gadget._netvibesSpecUrl = netvibesSpecUrl !=  '' ? netvibesSpecUrl : null;
	
	this.addGadget(gadget);
	
	if (userPrefs != null && gadget.userPrefs != null)
	{
		for (var name in userPrefs)
		{ 
			if (gadget.userPrefs[name])
			{
				gadget.userPrefs[name].value = userPrefs[name];
			}
		}
		gadget.refresh();
	}
};

AmetysContainer.ajustIFrameHeight = function (iframeID)
{
	try
	{
		var frame = document.getElementById(iframeID) != null ? document.getElementById(iframeID) : document.frames[iframeID];
		var frameDoc = frame.contentDocument ? frame.contentDocument : document.frames[iframeID].document;
		frame.height = frameDoc.body.offsetHeight;
	}
	catch (e){};
}

/**
 * Generates a secure token from a gadget url
 * @param gadgetUrl the gadget url
 * @return the secure token
 */
AmetysContainer.prototype.generateSecureToken = function (gadgetUrl, viewerId, ownerId) 
{
	var appId = 0;
	for (var i = 0; i < gadgetUrl.length; i++) 
	{
		appId += gadgetUrl.charCodeAt(i);
	}
	
	var fields = [ ownerId, viewerId, appId, "shindig", gadgetUrl, "0", "default" ];
	for (var i = 0; i < fields.length; i++) 
	{
		// escape each field individually, for metachars in URL
		fields[i] = escape(fields[i]);
	}
	
	return fields.join(":");
}

/**
 * Get the gadget metadata
 * @param gadgetUrl the gadget url
 * @param opt_callback the callback function
 */
AmetysContainer.prototype.requestGadgetMetaData = function (gadgetUrl, opt_callback)
{
	var request = {
		context : {
			language : shindig.container.language_, 
			view : shindig.container.view_,
			container : "ametys"
		},
		gadgets : [ {
			url : gadgetUrl,
			moduleId: 1
		} ]
	};
	this.sendRequestToServer(context.contextPath + '/plugins/gadgets/' + "metadata", "POST", gadgets.json.stringify(request), opt_callback, true);
}

/**
 * Send request
 * @param url
 * @param method
 * @param opt_postParams
 * @param opt_callback
 * @param opt_excludeSecurityToken
 * @return
 */
AmetysContainer.prototype.sendRequestToServer = function (url, method, opt_postParams, opt_callback, opt_excludeSecurityToken) 
{
	opt_postParams = opt_postParams || {};

	var makeRequestParams = {
		"CONTENT_TYPE" : "JSON",
		"METHOD" : method,
		"POST_DATA" : opt_postParams
	};

	if (!opt_excludeSecurityToken) 
	{
		url = socialDataPath + url + "?st=" + gadget.secureToken;
	}

	gadgets.io.makeNonProxiedRequest(url, function(data) {
		data = data.data;
		if (opt_callback) {
			opt_callback(data);
		}
	}, makeRequestParams, "application/javascript");
};

/**
 * Generates gadgets from the given metadata
 * @param metadata the gadgets' metadata
 */
AmetysContainer.prototype.generateGadgets = function (metadata, layout, allowDeletion, allowToggle, callback) 
{
	for (var i = 0; i < metadata.gadgets.length; i++) 
	{
		if (metadata.gadgets[i].views != null && metadata.gadgets[i].views['default'] != null)
		{
			shindig.container.setView("default"); 
		}
		else
		{
			shindig.container.setView("home"); 
		}
		
		shindig.container.layoutManager = layout;
		
		var gadget = shindig.container.createGadget({
			'specUrl' : metadata.gadgets[i].url,
			'title' : metadata.gadgets[i].title,
			'userPrefs' : metadata.gadgets[i].userPrefs,
			'height': metadata.gadgets[i].height
		});
		
		var requiresPubSub2 = false;
	    var arr = metadata.gadgets[i].features;
	    if (arr != null)
	    {
		    for (var j = 0; j < arr.length; j++) {
		      if (arr[j] === 'pubsub-2') {
		        requiresPubSub2 = true;
		        break;
		      }
		    }
	    }
	    var subClass = requiresPubSub2 ? shindig.OAAIfrGadget : shindig.IfrGadget;
	    for (var name in subClass) if (subClass.hasOwnProperty(name)) {
	      gadget[name] = subClass[name];
	    }
	    
	    gadget.allowDeletion (allowDeletion);
	    gadget.allowToggle (allowToggle);
		
	    gadget._container = this;
	    
		shindig.container.addGadget(gadget);
	}
	
	shindig.container.renderGadget(gadget);
	
	// ajust iframe height (this do not work on cross domain)
	var iframeID = 'remote_iframe_' + gadget.id;
	var iframe = document.getElementById(iframeID) != null ? document.getElementById(iframeID) : document.frames[iframeID];
	iframe.setAttribute('onload', "AmetysContainer.ajustIFrameHeight('" + iframeID + "')");
	
	if (typeof callback == 'function')
	{
		callback.apply(this, [gadget]);
	}
};

AmetysContainer.prototype.drawIfr = function (metadata, iframeId, gadgetSpec)
{
	for (var i = 0; i < metadata.gadgets.length; i++) 
	{
		var gadget = shindig.container.createGadget({
			'specUrl' : metadata.gadgets[i].url,
			'title' : metadata.gadgets[i].title,
			'userPrefs' : metadata.gadgets[i].userPrefs
		});
		
		var subClass = shindig.IfrGadget;
			for (var name in subClass) if (subClass.hasOwnProperty(name)) {
				gadget[name] = subClass[name];
			}
	}
	
	if (gadgetSpec.userPrefs != null && gadget.userPrefs != null)
	{
		for (var name in gadgetSpec.userPrefs)
		{ 
			if (gadget.userPrefs[name])
			{
				gadget.userPrefs[name].value = gadgetSpec.userPrefs[name];
			}
		}
	}
	
	$j ('#' + iframeId)[0].src = gadget.getIframeUrl();
}